@@ -1,5 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-1998, 2000-2010, 2012-2013, 2015-2016, 2019-2020, 2023 Free Software Foundation, Inc.
+   Copyright (C) 1995-2025 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -76,8 +76,8 @@ enum format_type
   format_ycp
 };
 #define NFORMATS 31     /* Number of format_type enum values.  */
-extern DLL_VARIABLE const char *const format_language[NFORMATS];
-extern DLL_VARIABLE const char *const format_language_pretty[NFORMATS];
+extern LIBGETTEXTSRC_DLL_VARIABLE const char *const format_language[NFORMATS];
+extern LIBGETTEXTSRC_DLL_VARIABLE const char *const format_language_pretty[NFORMATS];
 
 /* Is current msgid a format string?  */
 enum is_format
@@ -127,7 +127,7 @@ enum syntax_check_type
   sc_bullet_unicode
 };
 #define NSYNTAXCHECKS 4
-extern DLL_VARIABLE const char *const syntax_check_name[NSYNTAXCHECKS];
+extern LIBGETTEXTSRC_DLL_VARIABLE const char *const syntax_check_name[NSYNTAXCHECKS];
 
 /* Is current msgid subject to a syntax check?  */
 #if 0
@@ -241,7 +241,8 @@ extern void
 extern void
        message_comment_dot_append (message_ty *mp, const char *comment);
 extern void
-       message_comment_filepos (message_ty *mp, const char *name, size_t line);
+       message_comment_filepos (message_ty *mp,
+                                const char *file_name, size_t line_number);
 extern message_ty *
        message_copy (message_ty *mp);
 
