@@ -1,5 +1,5 @@
 /* Fast fuzzy searching among messages.
-   Copyright (C) 2006, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2006.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -25,6 +25,7 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include "attribute.h"
 #include "xalloc.h"
 #include "po-charset.h"
 
@@ -130,7 +131,7 @@ addlast_index (index_list_ty list, index_ty idx)
 /* Add a given index to an index list.
    Return the new index list, if it had to be reallocated, or NULL if it
    didn't change.  */
-static inline index_list_ty
+MAYBE_UNUSED static inline index_list_ty
 add_index (index_list_ty list, index_ty idx)
 {
   index_list_ty result;
@@ -182,7 +183,12 @@ add_index (index_list_ty list, index_ty idx)
    limit the search to lengths l' in the range
      l / (2 / FUZZY_THRESHOLD - 1) <= l' <= l * (2 / FUZZY_THRESHOLD - 1)
    Thus we need the list of the short strings up to length:  */
-#define SHORT_MSG_MAX (int) (SHORT_STRING_MAX_BYTES * (2 / FUZZY_THRESHOLD - 1))
+#if !defined __SUNPRO_C
+# define SHORT_MSG_MAX (int) (SHORT_STRING_MAX_BYTES * (2 / FUZZY_THRESHOLD - 1))
+#else
+/* Sun C on Solaris 8 cannot compute this constant expression.  */
+# define SHORT_MSG_MAX 28
+#endif
 
 /* A fuzzy index contains a hash table mapping all n-grams to their
    occurrences list.  */
@@ -192,7 +198,7 @@ struct message_fuzzy_index_ty
   character_iterator_t iterator;
   hash_table gram4;
   size_t firstfew;
-  message_list_ty *short_messages[SHORT_MSG_MAX + 1];
+  message_list_ty **short_messages;
 };
 
 /* Allocate a fuzzy index corresponding to a given list of messages.
@@ -211,8 +217,7 @@ message_fuzzy_index_alloc (const message_list_ty *mlp,
   findex->iterator = po_charset_character_iterator (canon_charset);
 
   /* Setup hash table.  */
-  if (hash_init (&findex->gram4, 10 * count) < 0)
-    xalloc_die ();
+  hash_init (&findex->gram4, 10 * count);
   for (j = 0; j < count; j++)
     {
       message_ty *mp = mlp->item[j];
@@ -299,6 +304,7 @@ message_fuzzy_index_alloc (const message_list_ty *mlp,
     findex->firstfew = 10;
 
   /* Setup lists of short messages.  */
+  findex->short_messages = XNMALLOC (SHORT_MSG_MAX + 1, message_list_ty *);
   for (l = 0; l <= SHORT_MSG_MAX; l++)
     findex->short_messages[l] = message_list_alloc (false);
   for (j = 0; j < count; j++)
@@ -318,14 +324,14 @@ message_fuzzy_index_alloc (const message_list_ty *mlp,
   /* Shrink memory used by the lists of short messages.  */
   for (l = 0; l <= SHORT_MSG_MAX; l++)
     {
-      message_list_ty *mlp = findex->short_messages[l];
+      message_list_ty *smlp = findex->short_messages[l];
 
-      if (mlp->nitems < mlp->nitems_max)
+      if (smlp->nitems < smlp->nitems_max)
         {
-          mlp->nitems_max = mlp->nitems;
-          mlp->item =
+          smlp->nitems_max = smlp->nitems;
+          smlp->item =
             (message_ty **)
-            xrealloc (mlp->item, mlp->nitems_max * sizeof (message_ty *));
+            xrealloc (smlp->item, smlp->nitems_max * sizeof (message_ty *));
         }
     }
 
@@ -369,8 +375,6 @@ mult_index_list_accumulate (struct mult_index_list *accu, index_list_ty list)
   size_t len1 = accu->nitems;
   size_t len2 = list[IL_LENGTH];
   size_t need = len1 + len2;
-  struct mult_index *ptr1;
-  struct mult_index *ptr1_end;
   index_ty *ptr2;
   index_ty *ptr2_end;
   struct mult_index *destptr;
@@ -389,38 +393,44 @@ mult_index_list_accumulate (struct mult_index_list *accu, index_list_ty list)
     }
 
   /* Make a linear pass through accu and list simultaneously.  */
-  ptr1 = accu->item;
-  ptr1_end = ptr1 + len1;
   ptr2 = list + 2;
   ptr2_end = ptr2 + len2;
   destptr = accu->item2;
-  while (ptr1 < ptr1_end && ptr2 < ptr2_end)
+  if (len1 > 0)
     {
-      if (ptr1->index < *ptr2)
+      struct mult_index *ptr1;
+      struct mult_index *ptr1_end;
+
+      ptr1 = accu->item;
+      ptr1_end = ptr1 + len1;
+      while (ptr1 < ptr1_end && ptr2 < ptr2_end)
+        {
+          if (ptr1->index < *ptr2)
+            {
+              *destptr = *ptr1;
+              ptr1++;
+            }
+          else if (ptr1->index > *ptr2)
+            {
+              destptr->index = *ptr2;
+              destptr->count = 1;
+              ptr2++;
+            }
+          else /* ptr1->index == list[2 + i2] */
+            {
+              destptr->index = ptr1->index;
+              destptr->count = ptr1->count + 1;
+              ptr1++;
+              ptr2++;
+            }
+          destptr++;
+        }
+      while (ptr1 < ptr1_end)
         {
           *destptr = *ptr1;
           ptr1++;
+          destptr++;
         }
-      else if (ptr1->index > *ptr2)
-        {
-          destptr->index = *ptr2;
-          destptr->count = 1;
-          ptr2++;
-        }
-      else /* ptr1->index == list[2 + i2] */
-        {
-          destptr->index = ptr1->index;
-          destptr->count = ptr1->count + 1;
-          ptr1++;
-          ptr2++;
-        }
-      destptr++;
-    }
-  while (ptr1 < ptr1_end)
-    {
-      *destptr = *ptr1;
-      ptr1++;
-      destptr++;
     }
   while (ptr2 < ptr2_end)
     {
@@ -649,6 +659,7 @@ message_fuzzy_index_free (message_fuzzy_index_ty *findex)
   /* Free the short lists.  */
   for (l = 0; l <= SHORT_MSG_MAX; l++)
     message_list_free (findex->short_messages[l], 1);
+  free (findex->short_messages);
 
   /* Free the index lists occurring as values in the hash tables.  */
   iter = NULL;
